@@ -34,17 +34,37 @@
 #endif
 #endif
 
+#ifdef __VMS
+#define pthread_mutex_lock PTHREAD_MUTEX_LOCK
+#define pthread_mutex_unlock PTHREAD_MUTEX_UNLOCK
+#endif
+
 #ifndef __POSIX_NT__
 #  include <pthread.h>
 #else
+#  ifndef WIN32_LEAN_AND_MEAN
+#    define WIN32_LEAN_AND_MEAN
+#    define OMNI_DEFINED_WIN32_LEAN_AND_MEAN
+#  endif
 #  include <windows.h>
 #  include "pthread_nt.h"
+#  ifdef OMNI_DEFINED_WIN32_LEAN_AND_MEAN
+#    undef WIN32_LEAN_AND_MEAN
+#    undef OMNI_DEFINED_WIN32_LEAN_AND_MEAN
+#  endif
 #endif
+
 extern "C" void* omni_thread_wrapper(void* ptr);
 
 #define OMNI_MUTEX_IMPLEMENTATION			\
     pthread_mutex_t posix_mutex;
 
+#define OMNI_MUTEX_LOCK_IMPLEMENTATION                  \
+    pthread_mutex_lock(&posix_mutex);
+
+#define OMNI_MUTEX_UNLOCK_IMPLEMENTATION                \
+    pthread_mutex_unlock(&posix_mutex);
+
 #define OMNI_CONDITION_IMPLEMENTATION			\
     pthread_cond_t posix_cond;
 
