@@ -22,67 +22,68 @@
 //  USA.
 //
 
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <time.h>
-#include <iostream.h>
 #include <fcntl.h>
 #if defined(__VMS) && __VMS_VER < 70000000
-#include <omniVMS/unlink.hxx>
-#include <omniVms/utsname.hxx>
+#  include <omniVMS/unlink.hxx>
+#  include <omniVms/utsname.hxx>
 #endif
 #include <NamingContext_i.h>
 #include <ObjectBinding.h>
 #include <INSMapper.h>
 #include <log.h>
-#include <iomanip.h>
+
+#ifdef HAVE_STD
+#  include <iostream>
+#  include <iomanip>
+   using namespace std;
+#else
+#  include <iostream.h>
+#  include <iomanip.h>
+#endif
 
 #ifdef __WIN32__
-
-#include <io.h>
-#include <winbase.h>
-
-#define stat(x,y) _stat(x,y)
-#define unlink(x) _unlink(x)
-
+#  include <io.h>
+#  include <winbase.h>
+#  define stat(x,y) _stat(x,y)
+#  define unlink(x) _unlink(x)
 #else
-
-#include <unistd.h>
-#include <sys/utsname.h>
-
+#  include <unistd.h>
+#  include <sys/utsname.h>
 #endif
 
 #if defined(__nextstep__)
-#include <libc.h>
-#include <sys/param.h>
-#endif
-
-#if defined(__nextstep__)
-#include <sys/param.h>
+#  include <libc.h>
+#  include <sys/param.h>
 #endif
 
 #ifndef O_SYNC
-#ifdef  O_FSYNC              // FreeBSD 3.2 does not have O_SYNC???
-#define O_SYNC O_FSYNC
-#endif
+#  ifdef  O_FSYNC              // FreeBSD 3.2 does not have O_SYNC???
+#    define O_SYNC O_FSYNC
+#  endif
 #endif
 
-#if defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x500
-#define USE_STREAM_OPEN
-#define OPEN(name,mode,perm) open(name,mode,perm)
-#elif defined(__KCC)
-#define USE_STREAM_OPEN
-#define OPEN(name,mode,perm) open(name,mode)
-#elif defined(__GNUG__) && __GNUG__ >= 3
-#define USE_STREAM_OPEN
-#define OPEN(name,mode,perm) open(name,mode)
+#ifdef HAVE_STD
+#  define USE_STREAM_OPEN
+#  define OPEN(name,mode,perm) open(name,mode)
+#elif defined(__SUNPRO_CC) && __SUNPRO_CC >= 0x500
+#  define USE_STREAM_OPEN
+#  define OPEN(name,mode,perm) open(name,mode,perm)
+#elif defined(__DMC__)
+#  define USE_STREAM_OPEN
+#  define OPEN(name,mode,perm) open(name,mode,perm)
+#elif defined(__ICC)
+#  define USE_STREAM_OPEN
+#  define OPEN(name,mode,perm) open(name,mode,perm)
 #endif
 
-#ifdef _NO_STRDUP
-
+#ifndef HAVE_STRDUP
 
 // we have no strdup
 static char *
@@ -95,7 +96,7 @@ strdup (char* str)
     strcpy (newstr, str);
   return newstr;
 }
-#endif  // _NO_STRDUP
+#endif  // not HAVE_STRDUP
 
 
 static
@@ -178,7 +179,7 @@ omniNameslog::omniNameslog(int& p,char* logdir) : port(p)
     logdir[strlen(logdir)-1] = '\0';		// strip trailing '/'
   }
 
-#ifndef _USE_GETHOSTNAME
+#ifdef HAVE_UNAME
   struct utsname un;
   if (uname(&un) < 0) {
     cerr << ts.t() << "Error: cannot get the name of this host." << endl;
@@ -189,15 +190,16 @@ omniNameslog::omniNameslog(int& p,char* logdir) : port(p)
   char* logname = new char[strlen(logdir) + strlen("/omninames-")
 			   + strlen(un.nodename) + 1];
   sprintf(logname, "%s/omninames-%s", logdir, un.nodename);
-#else
+
+#elif HAVE_GETHOSTNAME
 
   // Apparently on some AIX versions, MAXHOSTNAMELEN is too small (32) to
   // reflect the true size a hostname can be. Check and fix the value.
 #ifndef MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN 256
+#  define MAXHOSTNAMELEN 256
 #elif   MAXHOSTNAMELEN < 64
-#undef  MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN 256
+#  undef  MAXHOSTNAMELEN
+#  define MAXHOSTNAMELEN 256
 #endif
 
   char hostname[MAXHOSTNAMELEN+1];
@@ -210,7 +212,8 @@ omniNameslog::omniNameslog(int& p,char* logdir) : port(p)
   char* logname = new char[strlen(logdir) + strlen("/omninames-")
 			   + strlen(hostname) + 1];
   sprintf(logname, "%s/omninames-%s", logdir, hostname);
-#endif // _USE_GETHOSTNAME
+
+#endif // HAVE_UNAME
 #elif defined(__WIN32__)
 
   // Get host name:
@@ -228,7 +231,8 @@ omniNameslog::omniNameslog(int& p,char* logdir) : port(p)
   sprintf(logname, "%s\\omninames-%s", logdir, machineName);
   
   delete[] machineName;
-#else
+
+#else // VMS
   char last(
     logdir[strlen(logdir)-1]
   );
@@ -302,7 +306,7 @@ omniNameslog::omniNameslog(int& p,char* logdir) : port(p)
     firstTime = 0;
 
 #ifdef __WIN32__
-    ifstream initf(active,ios::in | ios::nocreate);
+    ifstream initf(active,ios::in);
 #else
     ifstream initf(active);
 #endif
@@ -376,7 +380,6 @@ omniNameslog::init(CORBA::ORB_ptr          the_orb,
 	throw IOError();
       logf.attach(fd);
 #endif
-
       putPort(port, logf);
       reallyFlush(logf);
 
@@ -479,10 +482,20 @@ omniNameslog::init(CORBA::ORB_ptr          the_orb,
 
   {
     // Check to see if we need an INS forwarding agent
-    omniObjKey key;
-    rootContext->_getTheKey(key);
+    omniIOR_var ior;
+    ior = rootContext->_getIOR();
 
-    if (strncmp((const char*)key.key(), "NameService", 11)) {
+    IIOP::ProfileBody iiop;
+    const IOP::TaggedProfileList& profiles = ior->iopProfiles();
+    for (CORBA::ULong index = 0; index < profiles.length(); index++) {
+      if (profiles[index].tag == IOP::TAG_INTERNET_IOP) {
+	IIOP::unmarshalProfile(profiles[index],iiop);
+	break;
+      }
+    }
+
+    if (strncmp((const char*)iiop.object_key.get_buffer(),
+		"NameService", 11)) {
       cerr << ts.t() << "(Pre-INS log file)" << endl;
       new INSMapper(the_ins_poa, rootContext);
     }
@@ -542,6 +555,7 @@ omniNameslog::create(const PortableServer::ObjectId& id)
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
+      logf.clear();
       throw CORBA::PERSIST_STORE();
     }
     checkpointNeeded = 1;
@@ -558,6 +572,7 @@ omniNameslog::destroy(CosNaming::NamingContext_ptr nc)
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
+      logf.clear();
       throw CORBA::PERSIST_STORE();
     }
     checkpointNeeded = 1;
@@ -575,6 +590,7 @@ omniNameslog::bind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n,
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
+      logf.clear();
       throw CORBA::PERSIST_STORE();
     }
     checkpointNeeded = 1;
@@ -591,6 +607,7 @@ omniNameslog::unbind(CosNaming::NamingContext_ptr nc, const CosNaming::Name& n)
     } catch (IOError& ex) {
       cerr << ts.t() << flush;
       perror("I/O error writing log file");
+      logf.clear();
       throw CORBA::PERSIST_STORE();
     }
     checkpointNeeded = 1;
@@ -643,7 +660,6 @@ omniNameslog::checkpoint(void)
 
     ckpf.attach(fd);
 #endif
-
     putPort(port, ckpf);
 
     NamingContext_i* nci;
@@ -837,7 +853,7 @@ omniNameslog::getCreate(istream& file)
 
   getKey(id, file);
 
-  if (id.length() == 4) // SYS_ASSIGNED_ID_SIZE
+  if (id.length() == 12) // SYS_ASSIGNED_ID_SIZE + TRANSIENT_SUFFIX_SIZE
     rc = new NamingContext_i(poa, id, this);
   else
     rc = new NamingContext_i(ins_poa, id, this);
@@ -1052,12 +1068,7 @@ omniNameslog::putKey(const PortableServer::ObjectId& id, ostream& file)
 {
   file << hex;
   for (unsigned int i = 0; i < id.length(); i++) {
-#if !defined(__SUNPRO_CC) || __SUNPRO_CC < 0x500
     file << setfill('0') << setw(2) << (int)id[i];
-#else
-    // Eventually, use the following for all standard C++ compilers
-    file << std::setfill('0') << std::setw(2) << (int)id[i];
-#endif
   }
   file << dec;
 }
