@@ -11,9 +11,14 @@
 //        anyExample_clt.
 //
 
-#include <iostream.h>
 #include <anyExample.hh>
 
+#ifdef HAVE_STD
+#  include <iostream>
+   using namespace std;
+#else
+#  include <iostream.h>
+#endif
 
 class anyExample_i : public POA_anyExample {
 public:
@@ -41,8 +46,10 @@ CORBA::Any* anyExample_i::testOp(const CORBA::Any& a)
     cout << "Long: " << l << endl;
   }
 #ifndef NO_FLOAT
+  // XXX - should we provide stream ops for _CORBA_Double_ and
+  // _CORBA_Float_on VMS??
   else if (a >>= d) {
-    cout << "Double: " << d << endl;
+    cout << "Double: " << (double)d << endl;
   }
 #endif
   else if (a >>= str) {
@@ -69,7 +76,7 @@ CORBA::Any* anyExample_i::testOp(const CORBA::Any& a)
 int main(int argc, char** argv)
 {
   try {
-    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");
+    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
 
     CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
     PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);
