@@ -1,14 +1,19 @@
 //				Package : threadtests
 // prodcons.cc			Created : 6/95 tjr
 //
-// Copyright (C) Olivetti Research Ltd 1995
+// Copyright (C) AT&T Laboratories Cambridge 1995
 //
 // Demonstrates the use of condition variables for signalling between
 // two "producer" threads and three "consumer" threads.  Also demonstrates
 // the use of timed waits.
 //
 
-#include <iostream.h>
+#ifdef HAVE_STD
+#  include <iostream>
+   using namespace std;
+#else
+#  include <iostream.h>
+#endif
 #include <stdlib.h>
 #include <omnithread.h>
 
