@@ -1,15 +1,20 @@
 //				Package : threadtests
 // thrspecdata.cc		Created : 6/95 tjr
 //
-// Copyright (C) Olivetti Research Ltd 1995
+// Copyright (C) AT&T Laboratories Cambridge 1995
 //
 // Demonstrates the use of thread-specific data by having a class which
 // inherits from omni_thread.  Also demonstrates joining with a thread
 // which returns a value.
 //
 
-#include <iostream.h>
-#include "omnithread.h"
+#ifdef HAVE_STD
+#  include <iostream>
+   using namespace std;
+#else
+#  include <iostream.h>
+#endif
+#include <omnithread.h>
 
 class thread_with_data : public omni_thread {
     int my_thread_id_plus_two;
