@@ -29,6 +29,12 @@
 
 /*
   $Log: giopBiDir.cc,v $
+  Revision 1.1.2.20  2005/03/02 13:25:39  dgrisby
+  Variable name clash.
+
+  Revision 1.1.2.19  2005/01/29 17:25:03  dgrisby
+  Memory leaks in bidir connection management.
+
   Revision 1.1.2.18  2004/10/18 00:23:54  dgrisby
   Bug in trying to set up bidirectional GIOP with GIOP 1.0 and 1.1.
 
@@ -251,6 +257,18 @@ BiDirServerRope::BiDirServerRope(giopStrand* strand, giopAddress* addr) :
   strand->RopeLink::insert(pd_strands);
 }
 
+////////////////////////////////////////////////////////////////////////
+BiDirServerRope::~BiDirServerRope()
+{
+  giopAddressList::const_iterator i, last;
+  i    = pd_redirect_addresses.begin();
+  last = pd_redirect_addresses.end();
+  for (; i != last; i++) {
+    delete (*i);
+  }
+}
+
+
 ////////////////////////////////////////////////////////////////////////
 BiDirServerRope*
 BiDirServerRope::addRope(giopStrand* strand, const giopAddressList& addrlist) {
@@ -562,7 +580,8 @@ BiDirClientRope::acquireClient(const omniIOR* ior,
 /////////////////////////////////////////////////////////////////////////////
 static
 CORBA::Boolean
-getBiDirServiceContext(omniInterceptors::serverReceiveRequest_T::info_T& info) {
+getBiDirServiceContext(omniInterceptors::serverReceiveRequest_T::info_T& info)
+{
 
   if (!orbParameters::acceptBiDirectionalGIOP) {
     // XXX If the ORB policy is "don't support bidirectional", don't bother 
@@ -575,7 +594,8 @@ getBiDirServiceContext(omniInterceptors::serverReceiveRequest_T::info_T& info) {
   giopStrand& strand = (giopStrand&)((giopStream&)info.giop_s);
 
   if (ver.minor != 2 || strand.isClient()) {
-    // Only parse service context if the GIOP version is 1.2, on the server side
+    // Only parse service context if the GIOP version is 1.2, on the
+    // server side
     return 1;
   }
 
@@ -697,7 +717,13 @@ getBiDirServiceContext(omniInterceptors::serverReceiveRequest_T::info_T& info) {
 	}
 
 	BiDirServerRope* r = BiDirServerRope::addRope(&strand,addrList);
-
+      }
+      
+      giopAddressList::const_iterator addr, last;
+      addr = addrList.begin();
+      last = addrList.end();
+      for (; addr != last; addr++) {
+	delete (*addr);
       }
     }
   }
