@@ -29,6 +29,12 @@
 
 /*
   $Log: giopImpl12.cc,v $
+  Revision 1.1.4.29  2005/04/10 22:17:19  dgrisby
+  Fixes to connection management. Thanks Jon Biggar.
+
+  Revision 1.1.4.28  2005/02/02 00:21:07  dgrisby
+  Memory leak with CloseConnection or error received in a queued message.
+
   Revision 1.1.4.27  2004/10/18 00:23:54  dgrisby
   Bug in trying to set up bidirectional GIOP with GIOP 1.0 and 1.1.
 
@@ -212,11 +218,13 @@ private:
   giopImpl12& operator=(const giopImpl12&);
 };
 
-
 ////////////////////////////////////////////////////////////////////////
 void
 giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 
+  // On entry, this function owns the giopStream_Buffer. On exit, the
+  // buffer has either been assigned to another owner or deleted.
+
   unsigned char* hdr = (unsigned char*)b + b->start;
   GIOP::MsgType mtype = (GIOP::MsgType)hdr[7];
 
@@ -233,10 +241,14 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     // rather the equivalent place in GIOP 1.3+) due to receiving a
     // reply message on the receiver thread of a bidirectional
     // connection.
-    inputTerminalProtocolError(g, __FILE__, __LINE__);
-    // never reach here
-  }
 
+    // We accept a CloseConnection message with any GIOP version.
+    if ((GIOP::MsgType)hdr[7] != GIOP::CloseConnection) {
+      giopStream_Buffer::deleteBuffer(b);
+      inputTerminalProtocolError(g, __FILE__, __LINE__);
+      // never reach here
+    }
+  }
 
   CORBA::ULong reqid;
 
@@ -253,6 +265,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     }
   }
   else if ( mtype == GIOP::MessageError) {
+    giopStream_Buffer::deleteBuffer(b);
     inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here
   }
@@ -260,15 +273,17 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
     // orderly shutdown.
     CORBA::ULong minor;
     CORBA::Boolean retry;
+    giopStream_Buffer::deleteBuffer(b);
+    g->pd_strand->orderly_closed = 1;
     g->notifyCommFailure(0,minor,retry);
     g->pd_strand->state(giopStrand::DYING);
-    g->pd_strand->orderly_closed = 1;
     giopStream::CommFailure::_raise(minor,
 				    CORBA::COMPLETED_NO,
 				    retry,__FILE__,__LINE__);
     // never reach here
   }
   else {
+    giopStream_Buffer::deleteBuffer(b);
     inputTerminalProtocolError(g, __FILE__, __LINE__);
     // never reach here
   }
@@ -284,6 +299,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
   case GIOP::LocateReply:
     if (!(g->pd_strand->isClient() || g->pd_strand->biDir)) {
       omniTransportLock->unlock();
+      giopStream_Buffer::deleteBuffer(b);
       inputTerminalProtocolError(g, __FILE__, __LINE__);
       // never reach here
     }
@@ -304,6 +320,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	  if (target->inputFullyBuffered()) {
 	    // a reply has already been received!
 	    omniTransportLock->unlock();
+	    giopStream_Buffer::deleteBuffer(b);
 	    inputTerminalProtocolError(g, __FILE__, __LINE__);
 	    // never reach here
 	  }
@@ -312,6 +329,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    if (mtype != GIOP::Fragment) {
 	      // already got the header
 	      omniTransportLock->unlock();
+	      giopStream_Buffer::deleteBuffer(b);
 	      inputTerminalProtocolError(g, __FILE__, __LINE__);
 	      // never reach here
 	    }
@@ -319,6 +337,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	  else if (mtype == GIOP::Fragment) {
 	    // receive body before the header
 	    omniTransportLock->unlock();
+	    giopStream_Buffer::deleteBuffer(b);
 	    inputTerminalProtocolError(g, __FILE__, __LINE__);
 	    // never reach here
 	  }
@@ -344,6 +363,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
       {
 	if (g->pd_strand->isClient() && !g->pd_strand->biDir) {
 	  omniTransportLock->unlock();
+	  giopStream_Buffer::deleteBuffer(b);
 	  inputTerminalProtocolError(g, __FILE__, __LINE__);
 	  // never reach here
 	}
@@ -359,6 +379,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	  else if (target->requestId() == reqid) {
 	    // already have a request with the same id.
 	    omniTransportLock->unlock();
+	    giopStream_Buffer::deleteBuffer(b);
 	    inputTerminalProtocolError(g, __FILE__, __LINE__);
 	    // never reach here
 	  }
@@ -372,9 +393,11 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	unused->state(IOP_S::InputPartiallyBuffered);
 	unused->requestId(reqid);
       }
+      // falls through
     case GIOP::CancelRequest:
       if (g->pd_strand->isClient() && !g->pd_strand->biDir) {
 	omniTransportLock->unlock();
+	giopStream_Buffer::deleteBuffer(b);
 	inputTerminalProtocolError(g, __FILE__, __LINE__);
 	// never reach here
       }
@@ -395,6 +418,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
 	    if (target->inputFullyBuffered()) {
 	      // a reply has already been received!
 	      omniTransportLock->unlock();
+	      giopStream_Buffer::deleteBuffer(b);
 	      inputTerminalProtocolError(g, __FILE__, __LINE__);
 	      // never reach here
 	    }
@@ -448,10 +472,7 @@ giopImpl12::inputQueueMessage(giopStream* g,giopStream_Buffer* b) {
       if (!matched_target_is_client) {
 
 	((GIOP_S*)matched_target)->state(IOP_S::InputFullyBuffered);
-	if (omniORB::trace(40)) {
-	  omniORB::logger log;
-	  log << "Changed GIOP_S to InputFullyBuffered\n";
-	}
+	omniORB::logs(40, "Changed GIOP_S to InputFullyBuffered");
 	if (!g->pd_strand->isClient()) {
 	  g->pd_strand->server->notifyCallFullyBuffered(g->pd_strand->connection);
 	}
@@ -700,6 +721,7 @@ giopImpl12::inputMessageBegin(giopStream* g,
     GIOP::Version v;
     v.major = 1;
     v.minor = hdr[5];
+    ((giopStrand &)*g).version = v;
     g->impl(giopStreamImpl::matchVersion(v));
     OMNIORB_ASSERT(g->impl());
     g->impl()->inputMessageBegin(g,g->impl()->unmarshalWildCardRequestHeader);
