@@ -29,6 +29,24 @@
 
 /*
   $Log: giopServer.cc,v $
+  Revision 1.22.2.37  2005/04/10 22:17:19  dgrisby
+  Fixes to connection management. Thanks Jon Biggar.
+
+  Revision 1.22.2.36  2005/03/10 11:28:29  dgrisby
+  Race condition between setSelectable / clearSelectable.
+
+  Revision 1.22.2.35  2005/02/23 12:27:31  dgrisby
+  Another race in setSelectable with connection shutdown. Thanks Peter
+  Klotz.
+
+  Revision 1.22.2.34  2005/02/13 20:52:51  dgrisby
+  Change threadPoolWatchConnection parameter to be an integer rather
+  than a boolean. Value is the count of threads that can be handling a
+  connection when one decides to watch it.
+
+  Revision 1.22.2.33  2004/12/20 18:17:55  dgrisby
+  Avoid race condition on connection disconnection.
+
   Revision 1.22.2.32  2004/10/17 20:14:32  dgrisby
   Updated support for OpenVMS. Many thanks to Bruce Visscher.
 
@@ -173,14 +191,20 @@ CORBA::ULong   orbParameters::maxServerThreadPoolSize        = 100;
 //
 //   Valid values = (n >= 1) 
 
-CORBA::Boolean orbParameters::threadPoolWatchConnection      = 1;
-//   1 means that after dispatching an upcall in thread pool mode, the
-//   thread should watch the connection for a short time before
-//   returning to the pool. This leads to less thread switching for
-//   series of calls from a single client, but is less fair if there
-//   are concurrent clients.
+CORBA::ULong   orbParameters::threadPoolWatchConnection      = 1;
+//   After dispatching an upcall in thread pool mode, the thread that
+//   has just performed the call can watch the connection for a short
+//   time before returning to the pool. This leads to less thread
+//   switching for a series of calls from a single client, but is less
+//   fair if there are concurrent clients. The connection is watched
+//   if the number of threads concurrently handling the connection is
+//   <= the value of this parameter. i.e. if the parameter is zero,
+//   the connection is never watched; if it is 1, the last thread
+//   managing a connection watches it; if 2, the connection is still
+//   watched if there is one other thread still in an upcall for the
+//   connection, and so on.
 //
-//  Valid values = 0 or 1
+//  Valid values = (n >= 0)
 
 
 ////////////////////////////////////////////////////////////////////////////
@@ -645,6 +669,7 @@ giopServer::csInsert(giopConnection* conn)
 
 
   giopStrand* s = new giopStrand(conn,this);
+  s->version.major = 1; s->version.minor = 0;
   {
     ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,0);
     omni_tracedmutex_lock sync(*omniTransportLock);
@@ -889,7 +914,7 @@ giopServer::removeConnectionAndWorker(giopWorker* w)
     giopConnection* conn = w->strand()->connection;
 
     conn->pd_dying = 1; // From now on, the giopServer will not create
-    // any more workers to serve this connection.
+                        // any more workers to serve this connection.
 
     cs = csLocate(conn);
 
@@ -1003,24 +1028,14 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
       // If there are other workers for this connection, or there are
       // too many temporary workers, let this worker finish.
 
-      if (conn->pd_n_workers > 1 ||
+      if (conn->pd_n_workers > orbParameters::threadPoolWatchConnection ||
 	  pd_n_temporary_workers > orbParameters::maxServerThreadPoolSize) {
 
-	w->remove();
-	delete w;
-	conn->pd_n_workers--;
-	pd_n_temporary_workers--;
-
 	select_and_return = 1;
       }
     }
-    if (select_and_return) {
-      // Connection is selectable now
-      conn->setSelectable(1);
-      return 0;
-    }
 
-    if (orbParameters::threadPoolWatchConnection) {
+    if (!select_and_return) {
       // Call Peek(). This thread will be used for a short time to
       // monitor the connection. If the connection is available for
       // reading, the callback function peekCallBack is called. We can
@@ -1035,6 +1050,10 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
       }
     }
 
+    // If connection is selectable, it's selectable now
+    if (!conn->pd_dying)
+      conn->setSelectable(2);
+
     // Worker is no longer needed.
     CORBA::Boolean dying = 0;
     {
@@ -1060,10 +1079,6 @@ giopServer::notifyWkDone(giopWorker* w, CORBA::Boolean exit_on_error)
 	pd_n_temporary_workers--;
       }
     }
-
-    // Connection is selectable now
-    conn->setSelectable(1);
-
     return dying ? 1 : 0;
   }
 #ifndef __DECCXX
@@ -1362,24 +1377,24 @@ public:
 
   threadPoolWatchConnectionHandler() : 
     orbOptions::Handler("threadPoolWatchConnection",
-			"threadPoolWatchConnection = 0 or 1",
+			"threadPoolWatchConnection = n >= 0",
 			1,
-			"-ORBthreadPoolWatchConnection < 0 | 1 >") {}
+			"-ORBthreadPoolWatchConnection < n >= 0 >") {}
 
 
   void visit(const char* value,orbOptions::Source) throw (orbOptions::BadParam) {
 
-    CORBA::Boolean v;
-    if (!orbOptions::getBoolean(value,v)) {
+    CORBA::ULong v;
+    if (!orbOptions::getULong(value,v)) {
       throw orbOptions::BadParam(key(),value,
-				 orbOptions::expect_boolean_msg);
+				 orbOptions::expect_ulong_msg);
     }
     orbParameters::threadPoolWatchConnection = v;
   }
 
   void dump(orbOptions::sequenceString& result) {
-    orbOptions::addKVBoolean(key(),orbParameters::threadPoolWatchConnection,
-			     result);
+    orbOptions::addKVULong(key(),orbParameters::threadPoolWatchConnection,
+			   result);
   }
 };
 
@@ -1506,7 +1521,7 @@ OMNI_NAMESPACE_END(omni)
 //        - setSelectable
 //
 //     notifyWkDone()
-//        - setSelectable(1) if n worker == 1.
+//        - setSelectable(2) if n worker == 1.
 //
 
 ////////////////////////////////////////////////////////////////////////////
