@@ -28,6 +28,13 @@
 
 /*
   $Log: giopStrand.cc,v $
+  Revision 1.1.4.23  2005/04/10 22:17:18  dgrisby
+  Fixes to connection management. Thanks Jon Biggar.
+
+  Revision 1.1.4.22  2005/03/16 09:16:59  dgrisby
+  Previous change to CancelRequest handling accidentally broke
+  server-side connection shutdown.
+
   Revision 1.1.4.21  2004/10/17 21:48:40  dgrisby
   Support CancelRequest better.
 
@@ -467,7 +474,10 @@ giopStrand::acquireServer(giopWorker* w)
 
   sp->state(IOP_S::Idle);
   if (!sp->impl()) {
-    sp->impl(giopStreamImpl::maxVersion());
+    giopStreamImpl *impl = giopStreamImpl::maxVersion();
+    sp->impl(impl);
+    if (version.major == 0)
+	version = impl->version();
   }
   // the codeset convertors are filled in by the codeset interceptor
   // before a request is unmarshalled.
@@ -540,10 +550,12 @@ giopStrand::releaseServer(IOP_S* iop_s)
     giop_s->giopStreamList::insert(servers);
   }
 
-  if (remove && giop_s->state() != IOP_S::WaitingForReply)
-    delete giop_s;
-  else
-    restart_idle = 0;
+  if (remove) {
+    if (giop_s->state() != IOP_S::WaitingForReply)
+      delete giop_s;
+    else
+      restart_idle = 0;
+  }
 
   if (restart_idle && !biDir) {
     CORBA::Boolean success = startIdleCounter();
@@ -584,10 +596,10 @@ CORBA::Boolean
 giopStrand::startIdleCounter() {
   ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,1);
 
-  if (idlebeats >= 0) 
+  if (idlebeats >= 0) {
     // The idle counter is already active or has already expired.
     return 0;
-  
+  }
   if (isClient()) {
     idlebeats = (idleOutgoingBeats) ? (CORBA::Long)idleOutgoingBeats : -1;
   }
@@ -602,10 +614,10 @@ CORBA::Boolean
 giopStrand::stopIdleCounter() {
   ASSERT_OMNI_TRACEDMUTEX_HELD(*omniTransportLock,1);
 
-  if (idlebeats == 0)
+  if (idlebeats == 0) {
     // The idle counter has already expired.
     return 0;
-
+  }
   idlebeats = -1;
   return 1;
 }
@@ -727,10 +739,9 @@ Scavenger::execute()
 	}	
 	{
 	  // Send close connection message.
-	  GIOP::Version ver = giopStreamImpl::maxVersion()->version();
 	  char hdr[12];
 	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = ver.major;   hdr[5] = ver.minor;
+	  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
 	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
 	  hdr[7] = (char)GIOP::CloseConnection;
 	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
@@ -1010,10 +1021,9 @@ public:
 	}	
 	{
 	  // Send close connection message.
-	  GIOP::Version ver = giopStreamImpl::maxVersion()->version();
 	  char hdr[12];
 	  hdr[0] = 'G'; hdr[1] = 'I'; hdr[2] = 'O'; hdr[3] = 'P';
-	  hdr[4] = ver.major;   hdr[5] = ver.minor;
+	  hdr[4] = s->version.major;   hdr[5] = s->version.minor;
 	  hdr[6] = _OMNIORB_HOST_BYTE_ORDER_;
 	  hdr[7] = (char)GIOP::CloseConnection;
 	  hdr[8] = hdr[9] = hdr[10] = hdr[11] = 0;
