@@ -29,6 +29,13 @@
 
 /*
   $Log: omniInternal.cc,v $
+  Revision 1.2.2.38  2005/04/08 00:06:14  dgrisby
+  Remove all remaining uses of logf.
+
+  Revision 1.2.2.37  2005/03/29 14:41:33  dgrisby
+  Use omniORB logger instead of logf. May avoid a compiler / platform
+  bug on VxWorks.
+
   Revision 1.2.2.36  2004/10/17 20:14:33  dgrisby
   Updated support for OpenVMS. Many thanks to Bruce Visscher.
 
@@ -382,10 +389,10 @@ omniObjTable::resize()
   }
   CORBA::ULong newsize = newsizei;
 
-  if( omniORB::trace(15) ) {
-    omniORB::logf("Object table resizing from %lu to %lu",
-		  (unsigned long) objectTableSize,
-		  (unsigned long) newsize);
+  if (omniORB::trace(15)) {
+    omniORB::logger l;
+    l << "Object table resizing from " << objectTableSize
+      << " to " << newsize << "\n";
   }
 
   // Create and initialise new object table.
@@ -486,8 +493,10 @@ omni::releaseObjRef(omniObjRef* objref)
     objref->_setIdentity(0);
   }
 
-  if( omniORB::trace(15) )
-    omniORB::logf("ObjRef(%s) -- deleted.", objref->_mostDerivedRepoId());
+  if( omniORB::trace(15) ) {
+    omniORB::logger l;
+    l << "ObjRef(" << objref->_mostDerivedRepoId() << ") -- deleted.\n";
+  }
 
   // Destroy the reference.
   delete objref;
