@@ -26,18 +26,31 @@
 //   
 //   Scope manipulation
 
-// $Id: idlscope.cc,v 1.11.2.9 2001/02/20 11:25:12 dpg1 Exp $
+// $Id: idlscope.cc,v 1.13.2.9 2003/04/09 10:26:48 dgrisby Exp $
 // $Log: idlscope.cc,v $
-// Revision 1.11.2.9  2001/02/20 11:25:12  dpg1
-// Changes for Digital Unix 4.0E.
+// Revision 1.13.2.9  2003/04/09 10:26:48  dgrisby
+// Silly variable reuse bug in new CORBA 3 keyword check.
 //
-// Revision 1.11.2.8  2000/11/30 11:40:48  dpg1
-// Add -nc option to omniidl to accept invalid IDL with identifiers
-// differing only by case.
+// Revision 1.13.2.8  2003/03/20 10:24:27  dgrisby
+// Warn about use of CORBA 3 keywords in IDL.
 //
-// Revision 1.11.2.7  2000/10/24 09:53:30  dpg1
-// Clean up omniidl system dependencies. Replace use of _CORBA_ types
-// with IDL_ types.
+// Revision 1.13.2.7  2001/10/29 17:42:43  dpg1
+// Support forward-declared structs/unions, ORB::create_recursive_tc().
+//
+// Revision 1.13.2.6  2001/10/17 16:48:33  dpg1
+// Minor error message tweaks
+//
+// Revision 1.13.2.5  2001/06/08 17:12:23  dpg1
+// Merge all the bug fixes from omni3_develop.
+//
+// Revision 1.13.2.4  2000/12/05 17:45:19  dpg1
+// omniidl case sensitivity updates from omni3_develop.
+//
+// Revision 1.13.2.3  2000/10/27 16:31:10  dpg1
+// Clean up of omniidl dependencies and types, from omni3_develop.
+//
+// Revision 1.13.2.2  2000/10/10 10:18:51  dpg1
+// Update omniidl front-end from omni3_develop.
 //
 // Revision 1.11.2.6  2000/09/19 09:14:26  dpg1
 // Scope::Entry::Kind renamed to Scope::Entry::EntryKind to avoid
@@ -154,19 +167,19 @@ ScopedName::
 
 char*
 ScopedName::
-toString() const
+toString(IDL_Boolean qualify) const
 {
   int       i;
   Fragment* f;
 
-  i = absolute_ ? 2 : 0;
+  i = (qualify && absolute_) ? 2 : 0;
 
   for (f = scopeList_; f; f = f->next())
     i += strlen(f->identifier()) + 2;
 
   char* str = new char [i-1];
 
-  if (absolute_) {
+  if (qualify && absolute_) {
     str[0] = ':'; str[1] = ':';
     i = 2;
   }
@@ -680,13 +693,13 @@ findScopedName(const ScopedName* sn, const char* file, int line) const
 	  // Error -- ambiguous
 	  if (file) {
 	    char* ssn = sn->toString();
-	    IdlError(file, line, "Ambiguous name `%s':", ssn);
+	    IdlError(file, line, "Ambiguous name '%s':", ssn);
 	    delete [] ssn;
 
 	    for (; el; el = el->tail()) {
 	      char* ssn = el->head()->container()->scopedName()->toString();
 	      IdlErrorCont(el->head()->file(), el->head()->line(),
-			   "(`%s' defined in `%s')",
+			   "('%s' defined in '%s')",
 			   el->head()->identifier(), ssn);
 	      delete [] ssn;
 	    }
@@ -703,7 +716,7 @@ findScopedName(const ScopedName* sn, const char* file, int line) const
     if (!e) {
       if (file) {
 	char* ssn = sn->toString();
-	IdlError(file, line, "Error in look-up of `%s': `%s' not found",
+	IdlError(file, line, "Error in look-up of '%s': '%s' not found",
 		 ssn, fid);
 	delete [] ssn;
       }
@@ -714,11 +727,11 @@ findScopedName(const ScopedName* sn, const char* file, int line) const
       // Case clash
       if (file) {
 	char* ssn = sn->toString();
-	IdlError(file, line, "Error in look-up of `%s': `%s' differs in case",
+	IdlError(file, line, "Error in look-up of '%s': '%s' differs in case",
 		 ssn, fid);
 	delete [] ssn;
 	ssn = e->scopedName()->toString();
-	IdlErrorCont(e->file(), e->line(), "from `%s' declared here", ssn);
+	IdlErrorCont(e->file(), e->line(), "from '%s' declared here", ssn);
 	delete [] ssn;
       }
       return 0;
@@ -732,9 +745,9 @@ findScopedName(const ScopedName* sn, const char* file, int line) const
 	if (file) {
 	  char* ssn = sn->toString();
 	  IdlError(file, line,
-		   "Error in look-up of `%s': `%s' does not form a scope",
+		   "Error in look-up of '%s': '%s' does not form a scope",
 		   ssn, e->identifier());
-	  IdlErrorCont(e->file(), e->line(), "(`%s' defined here)",
+	  IdlErrorCont(e->file(), e->line(), "('%s' defined here)",
 		       e->identifier());
 	  delete [] ssn;
 	}
@@ -833,9 +846,9 @@ addUse(const ScopedName* sn, const char* file, int line)
     if (clash) {
       if (strcmp(id, clash->identifier())) {
 	char* ssn = sn->toString();
-	IdlError(file, line, "Use of `%s' clashes with identifier `%s'",
+	IdlError(file, line, "Use of '%s' clashes with identifier '%s'",
 		 ssn, clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(`%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "('%s' declared here)",
 		     clash->identifier());
 	delete [] ssn;
       }
@@ -871,19 +884,19 @@ addModule(const char* identifier, Scope* scope, Decl* decl,
 	  return; // Reopening the module
 	}
 	IdlError(file, line,
-		 "Declaration of module `%s' clashes with declaration "
-		 "of module `%s'", identifier, clash->identifier());
+		 "Declaration of module '%s' clashes with declaration "
+		 "of module '%s'", identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(module `%s' declared here)", clash->identifier());
+		     "(module '%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_DECL:
       {
 	IdlError(file, line,
-		 "Declaration of module `%s' clashes with declaration "
-		 "of %s `%s'", identifier, clash->decl()->kindAsString(),
+		 "Declaration of module '%s' clashes with declaration "
+		 "of %s '%s'", identifier, clash->decl()->kindAsString(),
 		 clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
@@ -897,10 +910,10 @@ addModule(const char* identifier, Scope* scope, Decl* decl,
     case Entry::E_PARENT:
       {
 	IdlError(file, line,
-		 "Declaration of module `%s' clashes with name of "
-		 "enclosing module `%s'", identifier, clash->identifier());
+		 "Declaration of module '%s' clashes with name of "
+		 "enclosing module '%s'", identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(module `%s' declared here)", clash->identifier());
+		     "(module '%s' declared here)", clash->identifier());
 	break;
       }
     }
@@ -928,43 +941,43 @@ addDecl(const char* identifier, Scope* scope, Decl* decl, IdlType* idltype,
     case Entry::E_MODULE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with declaration "
-		 "of module `%s'", decl->kindAsString(),
+		 "Declaration of %s '%s' clashes with declaration "
+		 "of module '%s'", decl->kindAsString(),
 		 identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(module `%s' declared here)", clash->identifier());
+		     "(module '%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_DECL:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with earlier declaration "
-		 "of %s `%s'", decl->kindAsString(), identifier,
+		 "Declaration of %s '%s' clashes with earlier declaration "
+		 "of %s '%s'", decl->kindAsString(), identifier,
 		 clash->decl()->kindAsString(), clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
     case Entry::E_CALLABLE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with %s `%s'",
+		 "Declaration of %s '%s' clashes with %s '%s'",
 		 decl->kindAsString(), identifier,
 		 clash->decl()->kindAsString(), clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
     case Entry::E_INHERITED:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with inherited %s `%s'",
+		 "Declaration of %s '%s' clashes with inherited %s '%s'",
 		 decl->kindAsString(), identifier,
 		 clash->decl()->kindAsString(), clash->identifier());
 	char* inhfrom =
 	  clash->inh_from()->container()->scopedName()->toString();
 	IdlErrorCont(clash->inh_from()->file(), clash->inh_from()->line(),
-		     "(`%s' declared in %s here)",
+		     "('%s' declared in %s here)",
 		     clash->identifier(), inhfrom);
 	delete [] inhfrom;
 	break;
@@ -972,29 +985,29 @@ addDecl(const char* identifier, Scope* scope, Decl* decl, IdlType* idltype,
     case Entry::E_INSTANCE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with instance `%s'",
+		 "Declaration of %s '%s' clashes with instance '%s'",
 		 decl->kindAsString(), identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(`%s' declared here)", clash->identifier());
+		     "('%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_USE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with use of "
-		 "identifier `%s'", decl->kindAsString(),
+		 "Declaration of %s '%s' clashes with use of "
+		 "identifier '%s'", decl->kindAsString(),
 		 identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(`%s' used here)", clash->identifier());
+		     "('%s' used here)", clash->identifier());
 	break;
       }
     case Entry::E_PARENT:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with name of "
-		 "enclosing scope `%s'", decl->kindAsString(),
+		 "Declaration of %s '%s' clashes with name of "
+		 "enclosing scope '%s'", decl->kindAsString(),
 		 identifier, clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(`%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "('%s' declared here)",
 		     clash->identifier());
 	break;
       }
@@ -1023,43 +1036,43 @@ addCallable(const char* identifier, Scope* scope, Decl* decl,
     case Entry::E_MODULE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with declaration "
-		 "of module `%s'", decl->kindAsString(),
+		 "Declaration of %s '%s' clashes with declaration "
+		 "of module '%s'", decl->kindAsString(),
 		 identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(module `%s' declared here)", clash->identifier());
+		     "(module '%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_DECL:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with earlier declaration "
-		 "of %s `%s'", decl->kindAsString(), identifier,
+		 "Declaration of %s '%s' clashes with earlier declaration "
+		 "of %s '%s'", decl->kindAsString(), identifier,
 		 clash->decl()->kindAsString(), clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
     case Entry::E_CALLABLE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with %s `%s'",
+		 "Declaration of %s '%s' clashes with %s '%s'",
 		 decl->kindAsString(), identifier,
 		 clash->decl()->kindAsString(), clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
     case Entry::E_INHERITED:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with inherited %s `%s'",
+		 "Declaration of %s '%s' clashes with inherited %s '%s'",
 		 decl->kindAsString(), identifier,
 		 clash->decl()->kindAsString(), clash->identifier());
 	char* inhfrom =
 	  clash->inh_from()->container()->scopedName()->toString();
 	IdlErrorCont(clash->inh_from()->file(), clash->inh_from()->line(),
-		     "(`%s' declared in %s here)",
+		     "('%s' declared in %s here)",
 		     clash->identifier(), inhfrom);
 	delete [] inhfrom;
 	break;
@@ -1067,29 +1080,29 @@ addCallable(const char* identifier, Scope* scope, Decl* decl,
     case Entry::E_INSTANCE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with instance `%s'",
+		 "Declaration of %s '%s' clashes with instance '%s'",
 		 decl->kindAsString(), identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(`%s' declared here)", clash->identifier());
+		     "('%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_USE:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with use of "
-		 "identifier `%s'", decl->kindAsString(),
+		 "Declaration of %s '%s' clashes with use of "
+		 "identifier '%s'", decl->kindAsString(),
 		 identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(`%s' used here)", clash->identifier());
+		     "('%s' used here)", clash->identifier());
 	break;
       }
     case Entry::E_PARENT:
       {
 	IdlError(file, line,
-		 "Declaration of %s `%s' clashes with name of "
-		 "enclosing scope `%s'", decl->kindAsString(),
+		 "Declaration of %s '%s' clashes with name of "
+		 "enclosing scope '%s'", decl->kindAsString(),
 		 identifier, clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(`%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "('%s' declared here)",
 		     clash->identifier());
 	break;
       }
@@ -1123,18 +1136,18 @@ addInherited(const char* id, Scope* scope, Decl* decl,
     case Entry::E_INHERITED:
       {
 	if (inh_from != clash->inh_from()) {
-	  IdlError(file, line, "In definition of `%s': clash between "
-		   "inherited identifiers `%s' and `%s'",
+	  IdlError(file, line, "In definition of '%s': clash between "
+		   "inherited identifiers '%s' and '%s'",
 		   identifier(), id, clash->identifier());
 	  char* inhfrom =
 	    inh_from->container()->scopedName()->toString();
 	  IdlErrorCont(inh_from->file(), inh_from->line(),
-		       "(%s `%s' declared in %s here)",
+		       "(%s '%s' declared in %s here)",
 		       decl->kindAsString(), id, inhfrom);
 	  delete [] inhfrom;
 	  inhfrom = clash->inh_from()->container()->scopedName()->toString();
 	  IdlErrorCont(clash->inh_from()->file(), clash->inh_from()->line(),
-		       "(%s `%s' declared in %s here)",
+		       "(%s '%s' declared in %s here)",
 		       clash->decl()->kindAsString(),
 		       clash->identifier(), inhfrom);
 	  delete [] inhfrom;
@@ -1147,10 +1160,10 @@ addInherited(const char* id, Scope* scope, Decl* decl,
 	// it's definitely illegal
 
 //  	IdlWarning(file, line,
-//  		   "Inherited %s `%s' clashes with interface name `%s'",
+//  		   "Inherited %s '%s' clashes with interface name '%s'",
 //  		   decl->kindAsString(), id, clash->identifier());
 //  	IdlWarningCont(decl->file(), decl->line(),
-//  		       "(%s `%s' declared here)", decl->kindAsString(), id);
+//  		       "(%s '%s' declared here)", decl->kindAsString(), id);
 	break;
       }
     }
@@ -1178,42 +1191,42 @@ addInstance(const char* identifier, Decl* decl, IdlType* idltype,
     case Entry::E_MODULE:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with declaration "
-		 "of module `%s'", identifier, clash->identifier());
+		 "Instance identifier '%s' clashes with declaration "
+		 "of module '%s'", identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(module `%s' declared here)", clash->identifier());
+		     "(module '%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_DECL:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with declaration "
-		 "of %s `%s'", identifier, clash->decl()->kindAsString(),
+		 "Instance identifier '%s' clashes with declaration "
+		 "of %s '%s'", identifier, clash->decl()->kindAsString(),
 		 clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
     case Entry::E_CALLABLE:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with %s `%s'",
+		 "Instance identifier '%s' clashes with %s '%s'",
 		 identifier, clash->decl()->kindAsString(),
 		 clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(%s `%s' declared here)",
+	IdlErrorCont(clash->file(), clash->line(), "(%s '%s' declared here)",
 		     clash->decl()->kindAsString(), clash->identifier());
 	break;
       }
     case Entry::E_INHERITED:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with inherited %s `%s'",
+		 "Instance identifier '%s' clashes with inherited %s '%s'",
 		 identifier, clash->decl()->kindAsString(),
 		 clash->identifier());
 	char* inhfrom =
 	  clash->inh_from()->container()->scopedName()->toString();
 	IdlErrorCont(clash->inh_from()->file(), clash->inh_from()->line(),
-		     "(`%s' declared in %s here)",
+		     "('%s' declared in %s here)",
 		     clash->identifier(), inhfrom);
 	delete [] inhfrom;
 	break;
@@ -1221,27 +1234,27 @@ addInstance(const char* identifier, Decl* decl, IdlType* idltype,
     case Entry::E_INSTANCE:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with instance `%s'",
+		 "Instance identifier '%s' clashes with instance '%s'",
 		 identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(`%s' declared here)", clash->identifier());
+		     "('%s' declared here)", clash->identifier());
 	break;
       }
     case Entry::E_USE:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with use of "
-		 "identifier `%s'", identifier, clash->identifier());
+		 "Instance identifier '%s' clashes with use of "
+		 "identifier '%s'", identifier, clash->identifier());
 	IdlErrorCont(clash->file(), clash->line(),
-		     "(`%s' used here)", clash->identifier());
+		     "('%s' used here)", clash->identifier());
 	break;
       }
     case Entry::E_PARENT:
       {
 	IdlError(file, line,
-		 "Instance identifier `%s' clashes with name of "
-		 "enclosing scope `%s'", identifier, clash->identifier());
-	IdlErrorCont(clash->file(), clash->line(), "(`%s' declared here)",
+		 "Instance identifier '%s' clashes with name of "
+		 "enclosing scope '%s'", identifier, clash->identifier());
+	IdlErrorCont(clash->file(), clash->line(), "('%s' declared here)",
 		     clash->identifier());
 	break;
       }
@@ -1281,28 +1294,52 @@ keywordClash(const char* identifier, const char* file, int line)
     "abstract", "any", "attribute", "boolean", "case", "char", "const",
     "context", "custom", "default", "double", "enum", "exception",
     "factory", "FALSE", "fixed", "float", "in", "inout", "interface",
-    "long", "module", "native", "Object", "octet", "oneway", "out",
-    "private", "public", "raises", "readonly", "sequence", "short",
-    "string", "struct", "supports", "switch", "TRUE", "truncatable",
-    "typedef", "union", "unsigned", "ValueBase", "valuetype", "void",
-    "wchar", "wstring", 0
+    "local", "long", "module", "native", "Object", "octet", "oneway",
+    "out", "private", "public", "raises", "readonly", "sequence",
+    "short", "string", "struct", "supports", "switch", "TRUE",
+    "truncatable", "typedef", "union", "unsigned", "ValueBase",
+    "valuetype", "void", "wchar", "wstring", 0
   };
 
-  for (const char** k = keywords; *k; k++) {
+  static const char* new_keywords[] = {
+    "component", "consumes", "emits", "eventtype", "finder", "getraises",
+    "home", "import", "multiple", "primarykey", "provides", "publishes",
+    "setraises", "typeid", "typeprefix", "uses", 0
+  };
+
+  const char** k;
+
+  for (k = keywords; *k; k++) {
     if (Config::caseSensitive) {
       if (!strcmp(*k, identifier)) {
-	IdlError(file, line, "Identifier `%s' is identical to keyword `%s'",
+	IdlError(file, line, "Identifier '%s' is identical to keyword '%s'",
 		 identifier, *k);
 	return 1;
       }
     }
     else {
       if (!strcasecmp(*k, identifier)) {
-	IdlError(file, line, "Identifier `%s' clashes with keyword `%s'",
+	IdlError(file, line, "Identifier '%s' clashes with keyword '%s'",
 		 identifier, *k);
 	return 1;
       }
     }
   }
+  for (k = new_keywords; *k; k++) {
+    if (Config::caseSensitive) {
+      if (!strcmp(*k, identifier)) {
+	IdlWarning(file, line, "Identifier '%s' is identical to "
+		   "CORBA 3 keyword '%s'.", identifier, *k);
+	return 1;
+      }
+    }
+    else {
+      if (!strcasecmp(*k, identifier)) {
+	IdlWarning(file, line, "Identifier '%s' clashes with "
+		   "CORBA 3 keyword '%s'", identifier, *k);
+	return 1;
+      }
+    }
+  }
   return 0;
 }
